# Direct Proxies API Shim for Node

This module contains a minimal wrapper around the reference shim implementation of the new Direct Proxies API that replaces the old Harmony Proxy proposal as of December 2011. It loads the shim unchanged but then wraps the Proxy, Proxy.create, and Proxy.createFunction functions in order to capture references to proxies and their targets. It then wrap Object.getOwnPropertyDescriptor and forges descriptors for lookups on target objects, presenting them as always
configurable. This preempts incompatabilities with the new API and the current implementation in V8 that would
cause errors to be thrown every time there's a non-configurable property.

The Direct-Proxies shim has recently been renamed to "harmony-reflect" and has moved to right here on GitHub. This is mainly thanks to the hard work of [Tom Van Cutsem](https://github.com/tvcutsem/harmony-reflect) in both producing the API to begin with along with a shim to provide the functionality on top of existing Proxy functionality.


The official documentation and API is here: [ES Wiki - Direct Proxies](http://wiki.ecmascript.org/doku.php?id=harmony:direct_proxies)



# Usage

The functionality of the shim has been minimized to the barest possible. It will load the shim and 

```javascript

var shim = require('direct-proxies');

//main global
var Proxy = shim('./harmony-reflect/reflect.js').Proxy;

//a vm context
var context = vm.createContext();
var Proxy = shim('./harmony-reflect/reflect.js', context).Proxy;

//create a context and shim it
var result = shim('./harmony-reflect/reflect.js', true);
//result.Proxy, result.context
v

// `Proxy` itself is now the creator for all proxies
var proxy = Proxy(target, handler);


// all traps now have their first parameter as `target` referencing the real object
{ getOwnPropertyDescriptor: function(target,name) }


// new global `Reflect` contains default traps
global.Reflect:
 { getOwnPropertyDescriptor,  // non-own `getProperty[Names|Descriptor]` gone
   getOwnPropertyNames,
   defineProperty,
   deleteProperty,            // renamed from `delete`
   freeze,                    // freeze/seal/preventExtensions separated
   seal,
   preventExtensions,
   has,
   hasOwn,
   get,
   set,
   enumerate,
   iterate,
   keys,
   apply,                     // apply/new are now traps on the handler
   construct }


// for virtualized objects you can use this for your handler.prototype
// it implements some handlers but requires the others, like old fundamental traps
global.Reflect.VirtualHandler.prototype
 abstract { getOwnPropertyDescriptor,
 abstract   getOwnPropertyNames,
 abstract   defineProperty,
 abstract   deleteProperty,
            freeze,
            seal,
 abstract   preventExtensions,
            has,
            hasOwn,
            get,
            set,
            enumerate,
            iterate,
            keys,
 abstract   apply,
            construct }
```


# Compatability

The current stable Node.js branch, 0.6.x, uses a version of V8 with bugs that makes it incompatible with this shim. Node's master branch (0.7.x unstable) uses a new V8 which resolves the issue and is **required** to make this work.


If you are using Node 0.7.x and are still getting "Illegal Access" errors I have a pull request that solves this issue (along with greatly improving util.inspect) here https://github.com/joyent/node/pull/2360. It should be integrated into Node's master branch soon and hopefully will be available along with the fixed V8 implementation for Node 0.8.
=======
If you are using Node >0.6.6 and are still getting "Illegal Access" errors see this:
https://github.com/joyent/node/pull/2109

# Trap Reference

```javascript
var traps = {
  getOwnPropertyDescriptor : ['target', 'name']                     , //->  desc | undefined
  getOwnPropertyNames      : ['target']                             , //->  [ string ]
  defineProperty           : ['target', 'name', 'descriptor']       , //->  boolean

  preventExtensions        : ['target']                              , //->  boolean
  freeze                   : ['target']                              , //->  boolean
  seal                     : ['target']                              , //->  boolean

  delete                   : ['target', 'name']                      , //->  boolean
  hasOwn                   : ['target', 'name']                      , //->  boolean
  has                      : ['target', 'name']                      , //->  boolean
  get                      : ['target', 'name', 'receiver']          , //->  any
  set                      : ['target', 'name', 'val', 'receiver']   , //->  boolean

  enumerate                : ['target']                               , //->  [ string ]
  iterate                  : ['target']                               , //->  iterator `next` fn
  keys                     : ['target']                               , //->  [ string ]

  apply                    : ['target', 'receiver', 'args']           , //->  any
  new                      : ['target', 'args']                       , //->  any
};
```